from __future__ import annotations

from .AFXMainWindow import AFXMainWindow
from .FXApp import FXApp


class AFXApp(FXApp):
    """This class is responsible for providing some high-level GUI control methods."""

    #: Used to query whether the GUI is locked.
    ID_QUERY_GUILOCK: int = hash("ID_QUERY_GUILOCK")

    #: Used to change the cursor.
    ID_SHOW_HOURGLASS: int = hash("ID_SHOW_HOURGLASS")

    def __init__(
        self,
        appName: str = "Abaqus/CAE",
        vendorName: str = "SIMULIA",
        productName: str = "''",
        majorNumber: int = -1,
        minorNumber: int = -1,
        updateNumber: int = -1,
        prerelease: bool = False,
    ):
        """Constructor.

        Parameters
        ----------
        appName : str
            Application registry key.
        vendorName : str
            Vendor registry key.
        productName : str
            Product name.
        majorNumber : int
            Version number.
        minorNumber : int
            Release number.
        updateNumber : int
            Update number.
        prerelease : bool
            Official/Prerelease flag.
        """

    def create(self):
        """Creates windows for the application.

        Reimplemented from FXApp.
        """

    def getAFXMainWindow(self) -> AFXMainWindow:  # type: ignore
        """Returns a pointer to the AFXMainWindow."""
        ...

    def getBasePrerelease(self) -> bool:  # type: ignore
        """Returns True if the base product is a prerelease."""
        ...

    def getBaseProductName(self) -> str:  # type: ignore
        """Returns the base product name."""
        ...

    def getBaseVersionNumbers(self, majorNumber: int, minorNumber: int, updateNumber: int):
        """Returns the base product's major, minor, and update numbers.

        Parameters
        ----------
        majorNumber : int
            Version number.
        minorNumber : int
            Release number.
        updateNumber : int
            Update number.
        """

    def getKernelInitializationCommand(self) -> str:  # type: ignore
        """Returns the command string that will be issued upon application startup."""
        ...

    def getPrerelease(self) -> bool:  # type: ignore
        """Returns True if this is a prerelease."""
        ...

    def getProductName(self) -> str:  # type: ignore
        """Returns the product name."""
        ...

    def getVersionNumbers(self) -> tuple[int, int, int]:  # type: ignore
        """Returns the major, minor, and update numbers."""
        ...

    def init(self, argc: int, argv: str, *args, **kwargs):
        """Initializes the application and connects to the kernel.

        Parameters
        ----------
        argc : int

        argv : str
        """

    def isLocked(self) -> bool:  # type: ignore
        """Returns True if the GUI is locked or False if otherwise.

        Reimplemented from FXApp.
        """
        ...

    def isProductCAE(self) -> bool:  # type: ignore
        """Returns True if the base product is Abaqus/CAE."""
        ...

    def isProductViewer(self) -> bool:  # type: ignore
        """Returns True if the base product is Abaqus/Viewer."""
        ...

    def isLearningEdition(self) -> bool:  # type: ignore
        """Returns True if the base product is a learning edition."""
        ...

    def lock(self):
        """Locks the GUI (normally used during command and mode processing)."""

    def run(self):
        """Runs the main application event loop until stop() is called,.

        Reimplemented from FXApp.
        """

    def runUntil(self, condition: int):
        """Run an event loop till some flag becomes non-zero. Reimplemented from FXApp.

        Parameters
        ----------
        condition : int
        """

    def unlock(self):
        """Unlocks the GUI."""



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc


@abaqus_class_doc
class DirectDampingByFrequencyComponent:
    """A DirectDampingByFrequencyComponent object is used to define direct damping over a range of frequencies.

    .. note::
        This object can be accessed by::

            import step
            mdb.models[name].steps[name].directDampingByFrequency.components[i]
    """

    #: A Float specifying the frequency value in cycles/time.
    frequency: float | None = None

    #: A Float specifying the fraction of critical damping.
    fraction: float | None = None



from __future__ import annotations

from .AFXGuiObjectManager import AFXGuiObjectManager
from .AFXToolsetGui import AFXToolsetGui
from .constants import DECOR_ALL
from .FXApp import FXApp
from .FXIcon import FXIcon
from .FXMainWindow import FXMainWindow


class AFXMainWindow(FXMainWindow):
    """This class is responsible for constructing the components of the main window.

    It also provides accessors for the various components constructed.
    """

    #: Used to handle the quit message.
    ID_QUIT: int = hash("ID_QUIT")

    #: Used to handle tabbing.
    ID_TAB: int = hash("ID_TAB")

    #: Toggle the visibility of the model tree.
    ID_TOGGLE_MODEL_TREE: int = hash("ID_TOGGLE_MODEL_TREE")

    #: Toggle full screen.
    ID_TOGGLE_FULL_SCREEN: int = hash("ID_TOGGLE_FULL_SCREEN")

    #: Do not use, do not delete; for use by derived classes.
    ID_LAST: int = hash("ID_LAST")

    def __init__(
        self,
        app: FXApp,
        title: str,
        icon: FXIcon | None = None,
        miniIcon: FXIcon | None = None,
        opts: int = DECOR_ALL,
        x: int = 0,
        y: int = 0,
        w: int = 0,
        h: int = 0,
    ):
        """Constructor.

        Parameters
        ----------
        app : FXApp
            FXApp object.
        title : str
            Main window title.
        icon : FXIcon | None
            Main window icon.
        miniIcon : FXIcon | None
            Minimized icon.
        opts : int
            Main window options.
        x : int
            X location of the main window.
        y : int
            Y location of the main window.
        w : int
            Width of the main window.
        h : int
            Height of the main window.
        """

    def appendApplicableModuleForTreeTab(self, name: str, moduleName: str):
        """Appends a module name to the list of modules to which a tree tab is applicable.

        Parameters
        ----------
        name : str
            Name of the tab item.
        moduleName : str
            Module name to be appended to the list of the tab's applicable modules.
        """

    def appendTreeTab(self, text: str, name: str):
        """Appends a new tab item to the tree toolset tab book and returns a vertical frame managed by the new
        tab item; you must call create() on the vertical frame after you construct all its child widgets.

        Parameters
        ----------
        text : str
            Text to be shown in the new tab item.
        name : str
            Name of the new tab item.
        """

    def appendVisibleModuleForTreeTab(self, name: str, moduleName: str):
        """Appends a module to the list of modules in which a tree tab is visible.

        Parameters
        ----------
        name : str
            Name of the tab item.
        moduleName : str
            Module name to be appended to the list of the tab's modules in which it is visible.
        """

    def create(self):
        """Virtual base class method for creating windowing system resources.

        Reimplemented from FXTopWindow.
        """

    def getContextBar(self):
        """Returns a pointer to the context bar container."""

    def getCurrentTreeTab(self):
        """Returns the current tab item."""

    def getDefaultHeight(self):
        """Returns the default main window height.

        Reimplemented from FXTopWindow.
        """

    def getDefaultWidth(self):
        """Returns the default main window width.

        Reimplemented from FXTopWindow.
        """

    def getDisplayedNameAtIndex(self, index: int):
        """Returns the displayed name at the given position in the list.

        Parameters
        ----------
        index : int
            Position in the module list.
        """

    def getDrawingAreaHeight(self):
        """Returns the height of the drawing area in pixels."""

    def getDrawingAreaWidth(self):
        """Returns the width of the drawing area in pixels."""

    def getHelpToolset(self):
        """Returns a pointer to the help toolset."""

    def getMenubar(self):
        """Returns a pointer to the menubar."""

    def getModule(self, name: str):
        """Returns the module specified by the given name argument.

        Parameters
        ----------
        name : str
            A String that specifies the module to get.
        """

    def getModuleName(self, displayedName: str):
        """Returns the module name for the given displayed name.

        Parameters
        ----------
        displayedName : str
            Displayed module name (English).
        """

    def getNumModules(self):
        """Returns the number of modules."""

    def getPluginToolset(self):
        """Returns the Plugin toolset."""

    def getSelectorFromFunction(self, function: str):
        """Returns the selector of the given shortcut function. Throws exception if not found.

        Parameters
        ----------
        function : str
            A String specifying the function as shown in the Customize dialog box.
        """

    def getTargetFromFunction(self, function: str):
        """Returns the target of the given shortcut function. Throws exception if not found.

        Parameters
        ----------
        function : str
            A String specifying the function as shown in the Customize dialog box.
        """

    def getToolbox(self):
        """Returns a pointer to the module toolbox container."""

    def getToolMenuPane(self):
        """Returns a pointer to the tools menu pane."""

    def getToolMenuTitle(self):
        """Returns a pointer to the Tools menu title."""

    def getToolset(self, name: str):
        """Returns the toolset specified by the given name argument.

        Parameters
        ----------
        name : str
            A String in the local language that specifies to toolset to get.
        """

    def getToolsetKernelInitializationCommands(self):
        """Returns the command string that should initialize the toolsets in the kernel that are corresponding
        to the toolsets registered with the main window."""

    def getWorkDirectory(self):
        """Returns the current working directory."""

    def hideCli(self):
        """Hides the command line interface."""

    def hideMessageArea(self):
        """Hides the message area interface."""

    def makeCustomToolsets(self):
        """This method has no base class implementation; it may be used by customizers to construct Abaqus/CAE
        toolsets or toolsets derived from Abaqus/CAE toolsets; constructing those toolsets in this method is
        necessary to insure that the toolset will be available to standard Abaqus/CAE modules that register that
        toolset, and to avoid creating duplicate widgets when the toolset is used by a custom toolset."""

    def registerHelpToolset(self, tool: AFXToolsetGui, opts: int):
        """Registers the Help toolset.

        Parameters
        ----------
        tool : AFXToolsetGui
            Pointer to toolset being registered.
        opts : int
            Options for creating toolset GUI components.
        """

    def registerModule(self, displayedName: str, moduleImportName: str, kernelInitializationCommand: str):
        """Registers a module to make it available in the module combo; also registers the initialization string
        to be sent to the kernel the first time the module is loaded.

        Parameters
        ----------
        displayedName : str
            The name that will be displayed in the Module combo box in the context bar.
        moduleImportName : str
            The name that will be used to import this module.
        kernelInitializationCommand : str
            The Python command sent to kernel when the module is loaded.
        """

    def registerToolset(self, tool: AFXGuiObjectManager, opts: int):
        """Registers a toolset that is always available in the main window.

        Parameters
        ----------
        tool : AFXGuiObjectManager
            Pointer to the object being registered.
        opts : int
            Options for creating toolset GUI components.
        """

    def setApplicabilityForTreeTab(self, name: str, moduleNames: str):
        """Sets the modules that are applicable to the given tree tab. When switching modules, if the current
        tab is applicable to the new module, it will remain current. When a tree tab is created, it is
        applicable to all modules--use this method to set the applicability to only certain modules.

        Parameters
        ----------
        name : str
            Name of the tab item.
        moduleNames : str
            A String containing module names separated by commas.
        """

    def setCurrentTreeTab(self, name: str):
        """Sets the tree toolset tab book's current tab item to the tab item specified by the given name.

        Parameters
        ----------
        name : str
            Name of the tab item to be set current.
        """

    def setVisibilityForTreeTab(self, name: str, moduleNames: str):
        """Sets the modules in which a tree tab is visible. When switching modules, if the tab has not been
        specified to be visible in the new module, the tab will be hidden; otherwise it will be shown. When a
        tree tab is created it is visible in all modules--use this method to set the visibility to only certain
        modules.

        Parameters
        ----------
        name : str
            Name of the tab item.
        moduleNames : str
            A String containing module names separated by commas.
        """

    def setWorkDirectory(self, directory: str):
        """Sets the current working directory.

        Parameters
        ----------
        directory : str
            A String specifying the new work directory.
        """

    def showCli(self):
        """Shows the command line interface."""

    def showMessageArea(self):
        """Shows the message area interface."""

    def writeToMessageArea(self, message: str):
        """Writes a string to the message area.

        Parameters
        ----------
        message : str
        """



from __future__ import annotations

from .constants import DEFAULT_PAD
from .FXIcon import FXIcon
from .FXLabel import FXLabel
from .FXTabBar import FXTabBar

#: Top side tabs.
TAB_TOP: int = hash("TAB_TOP")

#: Left side tabs.
TAB_LEFT: int = hash("TAB_LEFT")

#: Right side tabs.
TAB_RIGHT: int = hash("TAB_RIGHT")

#: Bottom side tabs.
TAB_BOTTOM: int = hash("TAB_BOTTOM")

#: Normal tab item style.
TAB_TOP_NORMAL: int = hash("TAB_TOP_NORMAL")


class FXTabItem(FXLabel):
    """A tab item is placed in a tab bar or tab book.

    When selected, the tab item sends a message to its parent, and causes itself to become the active tab,
    and raised slightly above the other tabs. In the tab book, activating a tab item also causes the
    corresponding panel to be raised to the top.
    """

    def __init__(
        self,
        p: FXTabBar,
        text: str,
        ic: FXIcon | None = None,
        opts: int = TAB_TOP_NORMAL,
        x: int = 0,
        y: int = 0,
        w: int = 0,
        h: int = 0,
        pl: int = 6,
        pr: int = 6,
        pt: int = DEFAULT_PAD,
        pb: int = DEFAULT_PAD,
    ):
        """Construct a tab item.

        Parameters
        ----------
        p : FXTabBar

        text : str

        ic : FXIcon | None

        opts : int

        x : int

        y : int

        w : int

        h : int

        pl : int

        pr : int

        pt : int

        pb : int
        """

    def canFocus(self):
        """Returns True because a tab item can receive focus.

        Reimplemented from FXWindow.
        """



from __future__ import annotations

from typing import Dict

from .ReferencePoint import ReferencePoint


class ReferencePoints(Dict[str, ReferencePoint]):
    def __getitem__(self, key: str) -> ReferencePoint:
        if key in self.keys():
            return self.get(key)  # type: ignore
        else:
            return ReferencePoint(point=(0.0, 0.0, 0.0))



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from .GeometricRestriction import GeometricRestriction


@abaqus_class_doc
class SizingClusterAreas(GeometricRestriction):
    """The SizingClusterAreas object defines a sizing cluster areas geometric restriction. The
    SizingClusterAreas object is derived from the GeometricRestriction object.

    .. note::
        This object can be accessed by::

            import optimization
            mdb.models[name].optimizationTasks[name].geometricRestrictions[name]
    """

    #: A String specifying the geometric restriction repository key.
    name: str

    #: tuple of Region objects specifying the regions to which the geometric restriction is
    #: applied.
    regions: tuple

    @abaqus_method_doc
    def __init__(self, name: str, regions: tuple):
        """This method creates a SizingClusterAreas object.

        .. note::
            This function can be accessed by::

                mdb.models[name].optimizationTasks[name].SizingClusterAreas

        Parameters
        ----------
        name
            A String specifying the geometric restriction repository key.
        regions
            Tuple of Region objects specifying the regions to which the geometric restriction is
            applied.

        Returns
        -------
        SizingClusterAreas
            A SizingClusterAreas object.
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the SizingClusterAreas object."""
        ...



"""Message maps are used by classes derived from FXObject (which includes most GUI classes) to route messages
sent to the object to a message handling method.

A typical use might be to route a message from a button in a dialog to one of the dialog's methods so that
some action is taken when the button is pressed.
"""

from __future__ import annotations

from typing import Callable

from .FXObject import FXObject


def FXMAPFUNC(object: FXObject, messageType: int, messageId: int, method: Callable):
    """Creates an entry in the object's message map that will route a message to a method.

    Parameters
    ----------
    object : FXObject
        An instance of the class in which the message map entry is to be made. Typically this is "self".
    messageType : int
        An integer flag specifying the message type (e.g. SEL_COMMAND).
    messageId : int
        An integer specifying the message ID.
    method : Callable
        The method to which the message is to be routed. This method must be specified by including the class name (e.g. MyDB.myMethod).
    """


def FXMAPFUNCS(object: FXObject, messageType: int, startMessageId: int, endMessageId: int, method: Callable):
    """Creates multiple entries in the object's message map that will route messages to a method.

    Parameters
    ----------
    object : FXObject
        An instance of the class in which the message map entry is to be made. Typically this is "self".
    messageType : int
        An integer flag specifying the message type (e.g. SEL_COMMAND).
    startMessageId : int
        An integer specifying the starting message ID.
    endMessageId : int
        An integer specifying the ending message ID.
    method : Callable
        The method to which the message is to be routed. This method must be specified by including the class name (e.g. MyDB.myMethod).
    """


def MKUINT(messageId: int, messageType: int):
    """Creates a message selector by combining a message ID and a message type.

    Parameters
    ----------
    messageId : int
        An integer specifying the message ID.
    messageType : int
        An integer flag specifying the message type (e.g. SEL_COMMAND).
    """


def SELID(selector: int):
    """Returns the message ID from a message selector.

    Parameters
    ----------
    selector : int
        A message selector.
    """


def SELTYPE(selector: int):
    """Returns the message type from a message selector.

    Parameters
    ----------
    selector : int
        A message selector.
    """



from __future__ import annotations

from typing import Sequence, Union

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..Material.Material import Material
from ..Region.Region import Region
from ..UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class SurfaceOffsetAssignment:
    """The SurfaceOffsetAssignment object stores the surface offset fraction assignment definition for surfaces
    in ContactExp and ContactStd objects. The SurfaceOffsetAssignment object has no constructor or members.

    .. note::
        This object can be accessed by::

            import interaction
            mdb.models[name].interactions[name].surfaceOffsetAssignments

        The corresponding analysis keywords are:

        - SURFACE PROPERTY ASSIGNMENT
    """

    @abaqus_method_doc
    def changeValuesInStep(self, stepName: str, index: int, value: Union[Literal[C.SPOS, C.ORIGINAL, C.SNEG], float]):
        """This method allows modification of surface offset fraction assignments already defined on surfaces in
        a given step.

        Parameters
        ----------
        stepName
            A String specifying the name of the step in which the surface offset assignments are to
            be modified.
        index
            An Int specifying the position of the surface offset fraction assignment whose value is
            to be modified.
        value
            A tuple specifying the value of the surface offset assignments for the surface whose
            index is referenced. Each tuple contains one entry:

            - A Float or a SymbolicConstant specifying the surface offset fraction value to be used
              for the surface. Possible values of the SymbolicConstant are ORIGINAL, SPOS, and SNEG.
        """
        ...

    @abaqus_method_doc
    def appendInStep(
        self,
        stepName: str,
        assignments: Sequence[tuple[Union[Region, Material, Literal[C.SPOS, C.ORIGINAL, C.SNEG, C.GLOBAL]], float]],
    ):
        """This method allows addition of surface offset fraction assignments to new surfaces in a given step.

        Parameters
        ----------
        stepName
            A String specifying the name of the step in which new surface offset fraction
            assignments are to be defined.
        assignments
            A sequence of tuples specifying the surface offset fraction assignments. Each tuple
            contains two entries:

            - A region object or the SymbolicConstant GLOBAL specifying the surface to which the
              offset fraction is assigned.
            - A Float or a SymbolicConstant specifying the surface offset fraction value to be used
              for the surface. Possible values of the SymbolicConstant are ORIGINAL, SPOS, and SNEG.
        """
        ...

    @abaqus_method_doc
    def delete(self, indices: tuple):
        """The delete method allows you to delete existing surface offset fraction assignments.

        Parameters
        ----------
        indices
            A sequence of Ints specifying the index of each surface offset fraction assignment to
            delete.
        """
        ...



"""GUI plug-in commands register GUI plug-ins either in the **Plug-ins** menu or in a toolbox."""
from typing import Sequence

from ..UtilityAndView.abaqusConstants import ALL, SymbolicConstant
from .AFXMode import AFXMode
from .FXIcon import FXIcon
from .FXObject import FXObject


def registerGuiMenuButton(
    toolboxName: str,
    object: FXObject,
    buttonText: str = "",
    messageId: int = AFXMode.ID_ACTIVATE,
    icon: FXIcon | None = None,
    kernelInitString: str = "",
    applicableModules: SymbolicConstant | Sequence[str] = ALL,
    version: str = "N/A",
    author: str = "N/A",
    description: str = "N/A",
    helpUrl: str = "N/A",
):
    """Registers a GUI plug-in in the **Plug-ins** menu.

    Parameters
    ----------
    toolboxName : str
        A String specifying the name of the toolbox in which the button will be shown. The name appears in the toolbox
        title bar.
    object : FXObject
        The GUI object to which a (messageId, SEL_COMMAND) message will be sent. The object must have been inherited
        from FXObject.
    buttonText : str, optional
        A String specifying the text to be displayed in the **Plug-ins** menu. Use a pipe ( | ) between words to specify
        submenus. The default value is the empty string.
    messageId : int, optional
        An Int specifying the ID to be used when sending a command to the GUI object. The default value is
        _AFXMode.ID_ACTIVATE_.
    icon : FXIcon, optional
        A FXIcon object specifying an icon to be displayed to the left of the text in the menu. For more information,
        see [afxCreateIcon](pt01ch01gob125.htm?contextscope=all#gui-auxiliary-afxcreateicon) in the
        “Auxiliary functions” section of this guide. The default value is None.
    kernelInitString : str, optional
        A String specifying the string sent to the kernel the first time this plug-in is invoked. The string is intended
        to initialize the kernel (typically by importing modules) in preparation for commands that will be sent by this
        plug-in’s GUI. The default value is the empty string.
    applicableModules : SymbolicConstant | Sequence[str], optional
        The SymbolicConstant ALL or a sequence of one or more Strings specifying the list of modules to which this
        plug-in applies. If a plug-in is not applicable to a module, it will be hidden when the user switches into that
        module. Possible values of the Strings in the sequence are “Part”, “Property”, “Assembly”, “Step”,
        “Interaction”, “Load”, “Mesh”, “Job”, “Visualization”, and “Sketch”. The default value is ALL.
    version : str, optional
        A String specifying the version of the plug-in. The version is displayed in the **About Plug-ins** dialog box.
        The default value is “N/A”.
    author : str, optional
        A String specifying the author of the plug-in. The author is displayed in the **About Plug-ins** dialog box.
        The default value is “N/A”.
    description : str, optional
        A String specifying the description of the plug-in. The description is displayed in the **About Plug-ins**
        dialog box. The default value is “N/A”.
    helpUrl : str, optional
        A String specifying the universal resource locator (URL) that points to the help for this plug-in. This URL can
        be loaded in a web browser from the **View** button in the **About Plug-ins** dialog box. The default value is
        “N/A”.
    """


def registerGuiToolButton(
    toolboxName: str,
    object: FXObject,
    messageId: int = AFXMode.ID_ACTIVATE,
    buttonText: str = "",
    icon: FXIcon | None = None,
    kernelInitString: str = "",
    applicableModules: SymbolicConstant | Sequence[str] = ALL,
    version: str = "N/A",
    author: str = "N/A",
    description: str = "N/A",
    helpUrl: str = "N/A",
):
    """Registers a GUI plug-in in a toolbox.

    Parameters
    ----------
    toolboxName : str
        A String specifying the name of the toolbox in which the button will be shown. The name appears in the toolbox
        title bar.
    object : FXObject
        The GUI object to which a (messageId, SEL_COMMAND) message will be sent. The object must have been inherited
        from FXObject.
    messageId : int, optional
        An Int specifying the ID to be used when sending a command to the GUI object. The default value is
        _AFXMode.ID_ACTIVATE_.
    buttonText : str, optional
        A String specifying the text to be displayed in the **Plug-ins** menu. The default value is the empty string.
    icon : FXIcon, optional
        A FXIcon object specifying an icon to be displayed to the left of the text in the menu. For more information,
        see [afxCreateIcon](pt01ch01gob125.htm?contextscope=all#gui-auxiliary-afxcreateicon) in the
        “Auxiliary functions” section of this guide. The default value is None.
    kernelInitString : str, optional
        A String specifying the string sent to the kernel the first time this plug-in is invoked. The string is intended
        to initialize the kernel (typically by importing modules) in preparation for commands that will be sent by this
        plug-in’s GUI. The default value is the empty string.
    applicableModules : SymbolicConstant | Sequence[str], optional
        The SymbolicConstant ALL or a sequence of one or more Strings specifying the list of modules to which this
        plug-in applies. If a plug-in is not applicable to a module, it will be hidden when the user switches into that
        module. Possible values of the Strings in the sequence are “Part”, “Property”, “Assembly”, “Step”,
        “Interaction”, “Load”, “Mesh”, “Job”, “Visualization”, and “Sketch”. The default value is ALL.
    version : str, optional
        A String specifying the version of the plug-in. The version is displayed in the **About Plug-ins** dialog box.
        The default value is “N/A”.
    author : str, optional
        A String specifying the author of the plug-in. The author is displayed in the **About Plug-ins** dialog box.
        The default value is “N/A”.
    description : str, optional
        A String specifying the description of the plug-in. The description is displayed in the **About Plug-ins**
        dialog box. The default value is “N/A”.
    helpUrl : str, optional
        A String specifying the universal resource locator (URL) that points to the help for this plug-in. This URL can
        be loaded in a web browser from the **View** button in the **About Plug-ins** dialog box. The default value is
        “N/A”.
    """
